@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2017-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2017 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *
  *  Authors: Mickey Sola
  *
@@ -94,7 +94,7 @@ int onas_fan_checkowner(int pid, const struct optstruct *opts)
     return CHK_CLEAN;
 }
 
-int onas_scan(const char *fname, int fd, const char **virname, const struct cl_engine *engine, int options, int extinfo)
+int onas_scan(const char *fname, int fd, const char **virname, const struct cl_engine *engine, struct cl_scan_options *options, int extinfo)
 {
     int ret = 0;
     struct cb_context context;
@@ -105,7 +105,7 @@ int onas_scan(const char *fname, int fd, const char **virname, const struct cl_e
     context.virsize = 0;
     context.scandata = NULL;
  
-    ret = cl_scandesc_callback(fd, virname, NULL, engine, options, &context);
+    ret = cl_scandesc_callback(fd, fname, virname, NULL, engine, options, &context);
 
     if (ret) {
         if (extinfo && context.virsize)
