@@ -1,6 +1,6 @@
 /*
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
- *  Copyright (C) 2007-2013 Sourcefire, Inc.
+ *  Copyright (C) 2015, 2017-2018 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2007-2012 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
  *
@@ -68,8 +68,8 @@ int main(int argc, char **argv)
 	struct optstruct *opts;
 	const struct optstruct *opt;
 
-    if(check_flevel())
-	exit(2);
+    if (check_flevel())
+	    exit(2);
 
 #if !defined(_WIN32) && !defined(C_BEOS)
     sigemptyset(&sigset);
@@ -202,7 +202,7 @@ void help(void)
     mprintf("\n");
     mprintf("                       Clam AntiVirus: Scanner %s\n", get_version());
     printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
-    printf("           (C) 2019 Cisco Systems, Inc.\n");
+    printf("           (C) 2007-2018 Cisco Systems, Inc.\n");
     mprintf("\n");
     mprintf("    clamscan [options] [file/directory/-]\n");
     mprintf("\n");
@@ -251,13 +251,10 @@ void help(void)
     mprintf("    --structured-ssn-count=N             Min SSN count to generate a detect\n");
     mprintf("    --structured-cc-count=N              Min CC count to generate a detect\n");
     mprintf("    --scan-mail[=yes(*)/no]              Scan mail files\n");
-    mprintf("    --phishing-sigs[=yes(*)/no]          Signature-based phishing detection\n");
-    mprintf("    --phishing-scan-urls[=yes(*)/no]     URL-based phishing detection\n");
+    mprintf("    --phishing-sigs[=yes(*)/no]          Enable email signature-based phishing detection\n");
+    mprintf("    --phishing-scan-urls[=yes(*)/no]     Enable URL signature-based phishing detection\n");
+    mprintf("    --heuristic-alerts[=yes(*)/no]       Heuristic alerts\n");
     mprintf("    --heuristic-scan-precedence[=yes/no(*)] Stop scanning as soon as a heuristic match is found\n");
-    mprintf("    --phishing-ssl[=yes/no(*)]           Always block (flag) SSL mismatches in URLs (phishing module)\n");
-    mprintf("    --phishing-cloak[=yes/no(*)]         Always block (flag) cloaked URLs (phishing module)\n");
-    mprintf("    --partition-intersection[=yes/no(*)] Detect partition intersections in raw disk images using heuristics\n");
-    mprintf("    --algorithmic-detection[=yes(*)/no]  Algorithmic detection\n");
     mprintf("    --normalize[=yes(*)/no]              Normalize html, script, and text files. Use normalize=no for yara compatibility\n");
     mprintf("    --scan-pe[=yes(*)/no]                Scan PE files\n");
     mprintf("    --scan-elf[=yes(*)/no]               Scan ELF files\n");
@@ -268,10 +265,15 @@ void help(void)
     mprintf("    --scan-xmldocs[=yes(*)/no]           Scan xml-based document files\n");
     mprintf("    --scan-hwp3[=yes(*)/no]              Scan HWP3 files\n");
     mprintf("    --scan-archive[=yes(*)/no]           Scan archive files (supported by libclamav)\n");
-    mprintf("    --detect-broken[=yes/no(*)]          Try to detect broken executable files\n");
-    mprintf("    --block-encrypted[=yes/no(*)]        Block (flag) encrypted archives\n");
-    mprintf("    --block-macros[=yes/no(*)]           Block (flag) OLE2 files with VBA macros\n");
-    mprintf("    --block-max[=yes/no(*)]              Block (flag) files that exceed max file size, max scan size, or max recursion limit\n");
+    mprintf("    --alert-broken[=yes/no(*)]           Alert on broken executable files (PE & ELF)\n");
+    mprintf("    --alert-encrypted[=yes/no(*)]        Alert on encrypted archives and documents\n");
+    mprintf("    --alert-encrypted-archive[=yes/no(*)] Alert on encrypted archives\n");
+    mprintf("    --alert-encrypted-doc[=yes/no(*)]    Alert on encrypted documents\n");
+    mprintf("    --alert-macros[=yes/no(*)]           Alert on OLE2 files containing VBA macros\n");
+    mprintf("    --alert-exceeds-max[=yes/no(*)]      Alert on files that exceed max file size, max scan size, or max recursion limit\n");
+    mprintf("    --alert-phishing-ssl[=yes/no(*)]     Alert on emails containing SSL mismatches in URLs\n");
+    mprintf("    --alert-phishing-cloak[=yes/no(*)]   Alert on emails containing cloaked URLs\n");
+    mprintf("    --alert-partition-intersection[=yes/no(*)] Alert on raw DMG image files containing partition intersections\n");
     mprintf("    --nocerts                            Disable authenticode certificate chain verification in PE files\n");
     mprintf("    --dumpcerts                          Dump authenticode certificate chain in PE files\n");
     mprintf("\n");
