@@ -1,6 +1,6 @@
 /*
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
- *  Copyright (C) 2011-2013 Sourcefire, Inc.
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2011 Sourcefire, Inc.
  *
  *  Author: Török Edvin
  *
@@ -19,7 +19,7 @@
  *  MA 02110-1301, USA.
  */
 
-#include "cltypes.h"
+#include "clamav-types.h"
 struct arc4_state {
     /* really just 8 bit, but it is faster if reads are aligned */
     uint32_t S[256];
