@@ -1,8 +1,8 @@
 /*
  *  ClamAV bytecode internal API
  *
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
- *  Copyright (C) 2009-2013 Sourcefire, Inc.
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2009-2010 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
  *
@@ -36,7 +36,6 @@
 #include <ctype.h>
 
 #include "clamav.h"
-#include "cltypes.h"
 #include "clambc.h"
 #include "bytecode.h"
 #include "bytecode_priv.h"
@@ -530,7 +529,7 @@ int32_t cli_bcapi_extract_new(struct cli_bc_ctx *ctx, int32_t id)
             size_t csize = cli_get_container_size(cctx, -2);
             cli_set_container(cctx, ctx->containertype, csize);
         }
-        res = cli_magic_scandesc(ctx->outfd, cctx);
+        res = cli_magic_scandesc(ctx->outfd, ctx->tempfile, cctx);
         cctx->recursion--;
         if (res == CL_VIRUS) {
             ctx->virname = cli_get_last_virus(cctx);
@@ -1275,7 +1274,202 @@ uint32_t cli_bcapi_engine_dconf_level(struct cli_bc_ctx *ctx)
 uint32_t cli_bcapi_engine_scan_options(struct cli_bc_ctx *ctx)
 {
     cli_ctx *cctx = (cli_ctx*)ctx->ctx;
-    return cctx->options;
+    uint32_t options = CL_SCAN_RAW;
+    
+    if (cctx->options->general & CL_SCAN_GENERAL_ALLMATCHES)
+        options |= CL_SCAN_ALLMATCHES;
+    if (cctx->options->general & CL_SCAN_GENERAL_HEURISTICS)
+        options |= CL_SCAN_ALGORITHMIC;
+    if (cctx->options->general & CL_SCAN_GENERAL_COLLECT_METADATA)
+        options |= CL_SCAN_FILE_PROPERTIES;
+    if (cctx->options->general & CL_SCAN_GENERAL_HEURISTIC_PRECEDENCE)
+        options |= CL_SCAN_HEURISTIC_PRECEDENCE;
+
+    if (cctx->options->parse & CL_SCAN_PARSE_ARCHIVE)
+        options |= CL_SCAN_ARCHIVE;
+    if (cctx->options->parse & CL_SCAN_PARSE_ELF)
+        options |= CL_SCAN_ELF;
+    if (cctx->options->parse & CL_SCAN_PARSE_PDF)
+        options |= CL_SCAN_PDF;
+    if (cctx->options->parse & CL_SCAN_PARSE_SWF)
+        options |= CL_SCAN_SWF;
+    if (cctx->options->parse & CL_SCAN_PARSE_HWP3)
+        options |= CL_SCAN_HWP3;
+    if (cctx->options->parse & CL_SCAN_PARSE_XMLDOCS)
+        options |= CL_SCAN_XMLDOCS;
+    if (cctx->options->parse & CL_SCAN_PARSE_MAIL)
+        options |= CL_SCAN_MAIL;
+    if (cctx->options->parse & CL_SCAN_PARSE_OLE2)
+        options |= CL_SCAN_OLE2;
+    if (cctx->options->parse & CL_SCAN_PARSE_HTML)
+        options |= CL_SCAN_HTML;
+    if (cctx->options->parse & CL_SCAN_PARSE_PE)
+        options |= CL_SCAN_PE;
+    // if (cctx->options->parse & CL_SCAN_MAIL_URL)
+    //    options |= CL_SCAN_MAILURL; /* deprecated circa 2009 */
+
+    if (cctx->options->heuristic & CL_SCAN_HEURISTIC_BROKEN)
+        options |= CL_SCAN_BLOCKBROKEN;
+    if (cctx->options->heuristic & CL_SCAN_HEURISTIC_EXCEEDS_MAX)
+        options |= CL_SCAN_BLOCKMAX;
+    if (cctx->options->heuristic & CL_SCAN_HEURISTIC_PHISHING_SSL_MISMATCH)
+        options |= CL_SCAN_PHISHING_BLOCKSSL;
+    if (cctx->options->heuristic & CL_SCAN_HEURISTIC_PHISHING_CLOAK)
+        options |= CL_SCAN_PHISHING_BLOCKCLOAK;
+    if (cctx->options->heuristic & CL_SCAN_HEURISTIC_MACROS)
+        options |= CL_SCAN_BLOCKMACROS;
+    if ((cctx->options->heuristic & CL_SCAN_HEURISTIC_ENCRYPTED_ARCHIVE) || 
+        (cctx->options->heuristic & CL_SCAN_HEURISTIC_ENCRYPTED_DOC))
+        options |= CL_SCAN_BLOCKENCRYPTED;
+    if (cctx->options->heuristic & CL_SCAN_HEURISTIC_PARTITION_INTXN)
+        options |= CL_SCAN_PARTITION_INTXN;
+    if (cctx->options->heuristic & CL_SCAN_HEURISTIC_STRUCTURED)
+        options |= CL_SCAN_STRUCTURED;
+    if (cctx->options->heuristic & CL_SCAN_HEURISTIC_STRUCTURED_SSN_NORMAL)
+        options |= CL_SCAN_STRUCTURED_SSN_NORMAL;
+    if (cctx->options->heuristic & CL_SCAN_HEURISTIC_STRUCTURED_SSN_STRIPPED)
+        options |= CL_SCAN_STRUCTURED_SSN_STRIPPED;
+
+    if (cctx->options->mail & CL_SCAN_MAIL_PARTIAL_MESSAGE)
+        options |= CL_SCAN_PARTIAL_MESSAGE;
+
+    if (cctx->options->dev & CL_SCAN_DEV_COLLECT_SHA)
+        options |= CL_SCAN_INTERNAL_COLLECT_SHA;
+    if (cctx->options->dev & CL_SCAN_DEV_COLLECT_PERFORMANCE_INFO)
+        options |= CL_SCAN_PERFORMANCE_INFO;
+
+    return options;
+}
+
+uint32_t cli_bcapi_engine_scan_options_ex(struct cli_bc_ctx *ctx, const uint8_t *option_name, uint32_t name_len)
+{
+    uint32_t i = 0;
+    char *option_name_l = NULL;
+
+    if (ctx == NULL || option_name == NULL || name_len == 0) {
+        cli_warnmsg("engine_scan_options_ex: Invalid arguments!");
+        return 0;
+    }
+
+    cli_ctx *cctx = (cli_ctx*)ctx->ctx;
+    if (cctx == NULL || cctx->options == NULL) {
+        cli_warnmsg("engine_scan_options_ex: Invalid arguments!");
+        return 0;
+    }
+
+    option_name_l = malloc(name_len + 1);
+    for (i = 0; i < name_len; i++) {
+        option_name_l[0] = tolower(option_name[i]);
+    }
+    option_name_l[name_len] = '\0';
+
+    if (strncmp(option_name_l, "general", MIN(name_len, sizeof("general")))) {
+        if (cli_memstr(option_name_l, name_len, "allmatch", sizeof("allmatch"))) {
+            return (cctx->options->general & CL_SCAN_GENERAL_ALLMATCHES) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "collect metadata", sizeof("collect metadata"))) {
+            return (cctx->options->general & CL_SCAN_GENERAL_COLLECT_METADATA) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "heuristics", sizeof("heuristics"))) {
+            return (cctx->options->general & CL_SCAN_GENERAL_HEURISTICS) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "precedence", sizeof("precedence"))) {
+            return (cctx->options->general & CL_SCAN_GENERAL_HEURISTIC_PRECEDENCE) ? 1 : 0;
+        }
+        /* else unknown option */
+        return 0;
+    }
+    else if (strncmp(option_name_l, "parse", MIN(name_len, sizeof("parse")))) {
+        if (cli_memstr(option_name_l, name_len, "archive", sizeof("archive"))) {
+            return (cctx->options->parse & CL_SCAN_PARSE_ARCHIVE) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "elf", sizeof("elf"))) {
+            return (cctx->options->parse & CL_SCAN_PARSE_ELF) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "pdf", sizeof("pdf"))) {
+            return (cctx->options->parse & CL_SCAN_PARSE_PDF) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "swf", sizeof("swf"))) {
+            return (cctx->options->parse & CL_SCAN_PARSE_SWF) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "hwp3", sizeof("hwp3"))) {
+            return (cctx->options->parse & CL_SCAN_PARSE_HWP3) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "xmldocs", sizeof("xmldocs"))) {
+            return (cctx->options->parse & CL_SCAN_PARSE_XMLDOCS) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "mail", sizeof("mail"))) {
+            return (cctx->options->parse & CL_SCAN_PARSE_MAIL) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "ole2", sizeof("ole2"))) {
+            return (cctx->options->parse & CL_SCAN_PARSE_OLE2) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "html", sizeof("html"))) {
+            return (cctx->options->parse & CL_SCAN_PARSE_HTML) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "pe", sizeof("pe"))) {
+            return (cctx->options->parse & CL_SCAN_PARSE_PE) ? 1 : 0;
+        }
+        /* else unknown option */
+        return 0;
+    }
+    else if (strncmp(option_name_l, "heuristic", MIN(name_len, sizeof("heuristic")))) {
+        if (cli_memstr(option_name_l, name_len, "broken", sizeof("broken"))) {
+            return (cctx->options->heuristic & CL_SCAN_HEURISTIC_BROKEN) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "exceeds max", sizeof("exceeds max"))) {
+            return (cctx->options->heuristic & CL_SCAN_HEURISTIC_EXCEEDS_MAX) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "phishing ssl mismatch", sizeof("phishing ssl mismatch"))) {
+            return (cctx->options->heuristic & CL_SCAN_HEURISTIC_PHISHING_SSL_MISMATCH) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "phishing cloak", sizeof("phishing cloak"))) {
+            return (cctx->options->heuristic & CL_SCAN_HEURISTIC_PHISHING_CLOAK) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "macros", sizeof("macros"))) {
+            return (cctx->options->heuristic & CL_SCAN_HEURISTIC_MACROS) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "encrypted archive", sizeof("encrypted archive"))) {
+            return (cctx->options->heuristic & CL_SCAN_HEURISTIC_ENCRYPTED_ARCHIVE) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "encrypted doc", sizeof("encrypted doc"))) {
+            return (cctx->options->heuristic & CL_SCAN_HEURISTIC_ENCRYPTED_DOC) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "partition intxn", sizeof("partition intxn"))) {
+            return (cctx->options->heuristic & CL_SCAN_HEURISTIC_PARTITION_INTXN) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "structured", sizeof("structured"))) {
+            return (cctx->options->heuristic & CL_SCAN_HEURISTIC_STRUCTURED) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "structured ssn normal", sizeof("structured ssn normal"))) {
+            return (cctx->options->heuristic & CL_SCAN_HEURISTIC_STRUCTURED_SSN_NORMAL) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "structured ssn stripped", sizeof("structured ssn stripped"))) {
+            return (cctx->options->heuristic & CL_SCAN_HEURISTIC_STRUCTURED_SSN_STRIPPED) ? 1 : 0;
+        }
+        /* else unknown option */
+        return 0;
+    }
+    else if (strncmp(option_name_l, "mail", MIN(name_len, sizeof("mail")))) {
+        if (cli_memstr(option_name_l, name_len, "partial message", sizeof("partial message"))) {
+            return (cctx->options->mail & CL_SCAN_MAIL_PARTIAL_MESSAGE) ? 1 : 0;
+        }
+        /* else unknown option */
+        return 0;
+    }
+    else if (strncmp(option_name_l, "dev", MIN(name_len, sizeof("dev")))) {
+        if (cli_memstr(option_name_l, name_len, "collect sha", sizeof("collect sha"))) {
+            return (cctx->options->dev & CL_SCAN_DEV_COLLECT_SHA) ? 1 : 0;
+        }
+        if (cli_memstr(option_name_l, name_len, "collect performance info", sizeof("collect performance info"))) {
+            return (cctx->options->dev & CL_SCAN_DEV_COLLECT_PERFORMANCE_INFO) ? 1 : 0;
+        }
+        /* else unknown option */
+        return 0;
+    } else {
+        /* else unknown option */
+        return 0;
+    }
 }
 
 uint32_t cli_bcapi_engine_db_options(struct cli_bc_ctx *ctx)
@@ -1427,7 +1621,7 @@ uint32_t cli_bcapi_check_platform(struct cli_bc_ctx *ctx , uint32_t a, uint32_t
 
 int cli_bytecode_context_setpdf(struct cli_bc_ctx *ctx, unsigned phase,
                                 unsigned nobjs,
-                                struct pdf_obj *objs, uint32_t *pdf_flags,
+                                struct pdf_obj **objs, uint32_t *pdf_flags,
                                 uint32_t pdfsize, uint32_t pdfstartoff)
 {
     ctx->pdf_nobjs = nobjs;
@@ -1470,7 +1664,7 @@ int32_t cli_bcapi_pdf_lookupobj(struct cli_bc_ctx *ctx , uint32_t objid)
     if (!ctx->pdf_phase)
         return -1;
     for (i=0;i<ctx->pdf_nobjs;i++) {
-        if (ctx->pdf_objs[i].id == objid)
+        if (ctx->pdf_objs[i]->id == objid)
             return i;
     }
     return -1;
@@ -1484,8 +1678,8 @@ uint32_t cli_bcapi_pdf_getobjsize(struct cli_bc_ctx *ctx , int32_t objidx)
        )
         return 0;
     if ((uint32_t)(objidx + 1) == ctx->pdf_nobjs)
-        return ctx->pdf_size - ctx->pdf_objs[objidx].start;
-    return ctx->pdf_objs[objidx+1].start - ctx->pdf_objs[objidx].start - 4;
+        return ctx->pdf_size - ctx->pdf_objs[objidx]->start;
+    return ctx->pdf_objs[objidx+1]->start - ctx->pdf_objs[objidx]->start - 4;
 }
 
 const uint8_t* cli_bcapi_pdf_getobj(struct cli_bc_ctx *ctx , int32_t objidx, uint32_t amount)
@@ -1493,7 +1687,7 @@ const uint8_t* cli_bcapi_pdf_getobj(struct cli_bc_ctx *ctx , int32_t objidx, uin
     uint32_t size = cli_bcapi_pdf_getobjsize(ctx, objidx);
     if (amount > size)
         return NULL;
-    return fmap_need_off(ctx->fmap, ctx->pdf_objs[objidx].start, amount);
+    return fmap_need_off(ctx->fmap, ctx->pdf_objs[objidx]->start, amount);
 }
 
 int32_t cli_bcapi_pdf_getobjid(struct cli_bc_ctx *ctx , int32_t objidx)
@@ -1501,7 +1695,7 @@ int32_t cli_bcapi_pdf_getobjid(struct cli_bc_ctx *ctx , int32_t objidx)
     if (!ctx->pdf_phase ||
         (uint32_t)objidx >= ctx->pdf_nobjs)
         return -1;
-    return ctx->pdf_objs[objidx].id;
+    return ctx->pdf_objs[objidx]->id;
 }
 
 int32_t cli_bcapi_pdf_getobjflags(struct cli_bc_ctx *ctx , int32_t objidx)
@@ -1509,7 +1703,7 @@ int32_t cli_bcapi_pdf_getobjflags(struct cli_bc_ctx *ctx , int32_t objidx)
     if (!ctx->pdf_phase ||
         (uint32_t)objidx >= ctx->pdf_nobjs)
         return -1;
-    return ctx->pdf_objs[objidx].flags;
+    return ctx->pdf_objs[objidx]->flags;
 }
 
 int32_t cli_bcapi_pdf_setobjflags(struct cli_bc_ctx *ctx , int32_t objidx, int32_t flags)
@@ -1518,9 +1712,9 @@ int32_t cli_bcapi_pdf_setobjflags(struct cli_bc_ctx *ctx , int32_t objidx, int32
         (uint32_t)objidx >= ctx->pdf_nobjs)
         return -1;
     cli_dbgmsg("cli_pdf: bytecode setobjflags %08x -> %08x\n",
-               ctx->pdf_objs[objidx].flags,
+               ctx->pdf_objs[objidx]->flags,
                flags);
-    ctx->pdf_objs[objidx].flags = flags;
+    ctx->pdf_objs[objidx]->flags = flags;
     return 0;
 }
 
@@ -1529,7 +1723,7 @@ int32_t cli_bcapi_pdf_get_offset(struct cli_bc_ctx *ctx , int32_t objidx)
     if (!ctx->pdf_phase ||
         (uint32_t)objidx >= ctx->pdf_nobjs)
         return -1;
-    return ctx->pdf_startoff + ctx->pdf_objs[objidx].start;
+    return ctx->pdf_startoff + ctx->pdf_objs[objidx]->start;
 }
 
 int32_t cli_bcapi_pdf_get_phase(struct cli_bc_ctx *ctx)
