@@ -1,6 +1,6 @@
 /*
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
- *  Copyright (C) 2007-2013 Sourcefire, Inc.
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2007-2009 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
  *
@@ -29,7 +29,6 @@
 #include "clamav.h"
 #include "memory.h"
 #include "others.h"
-#include "cltypes.h"
 #include "matcher.h"
 #include "matcher-bm.h"
 #include "filetypes.h"
@@ -381,7 +380,7 @@ int cli_bm_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 		    }
 		    if(virname) {
 			*virname = p->virname;
-			if(ctx != NULL && SCAN_ALL) {
+			if(ctx != NULL && SCAN_ALLMATCHES) {
 			    cli_append_virus(ctx, *virname);
 			    //*viroffset = offset + i + j - BM_MIN_LENGTH + BM_BLOCK_SIZE;
 			}
@@ -391,7 +390,7 @@ int cli_bm_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 
 		    viruses_found = 1;
 
-		    if(ctx != NULL && !SCAN_ALL)
+		    if(ctx != NULL && !SCAN_ALLMATCHES)
 			return CL_VIRUS;
 		}
 		p = p->next;
