@@ -1,6 +1,6 @@
 /*
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
- *  Copyright (C) 2008-2013 Sourcefire, Inc.
+ *  Copyright (C) 2015-2017 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2008 Sourcefire, Inc.
  *
  *  Authors: aCaB <acab@clamav.net>
  *
@@ -23,7 +23,7 @@
 #define MPOOL_H
 
 #ifdef USE_MPOOL
-#include "cltypes.h"
+#include "clamav-types.h"
 typedef struct MP mpool_t;
 struct cl_engine;
 
