@@ -1,6 +1,6 @@
 /*
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
- *  Copyright (C) 2007-2013 Sourcefire, Inc.
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2007-2008 Sourcefire, Inc.
  *
  *  Authors: Alberto Wu
  *
@@ -22,7 +22,7 @@
 #ifndef __REBUILDPE_H
 #define __REBUILDPE_H
 
-#include "cltypes.h"
+#include "clamav-types.h"
 #include "execs.h"
 
 int cli_rebuildpe(char *, struct cli_exe_section *, int, uint32_t, uint32_t, uint32_t, uint32_t, int);
