@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2011-2013 Sourcefire, Inc.
  *
  *  The code is based on Flasm, command line assembler & disassembler of Flash
@@ -45,7 +45,6 @@
 #include <time.h>
 #include <zlib.h>
 
-#include "cltypes.h"
 #include "swf.h"
 #include "clamav.h"
 #include "scanners.h"
@@ -286,7 +285,7 @@ static int scanzws(cli_ctx *ctx, struct swf_file_hdr *hdr)
                    hdr->filesize, (long long unsigned)outsize);
     }
 
-    ret = cli_magic_scandesc(fd, ctx);
+    ret = cli_magic_scandesc(fd, tmpname, ctx);
 
     close(fd);
     if (!(ctx->engine->keeptmp)) {
@@ -417,7 +416,7 @@ static int scancws(cli_ctx *ctx, struct swf_file_hdr *hdr)
                    hdr->filesize, (long long unsigned)outsize);
     }
 
-    ret = cli_magic_scandesc(fd, ctx);
+    ret = cli_magic_scandesc(fd, tmpname, ctx);
 
     close(fd);
     if(!ctx->engine->keeptmp) {
