@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2015-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *
  *  Authors: Kevin Lin <kevlin2@cisco.com>
  *
@@ -146,7 +146,7 @@ int cli_parsetiff(cli_ctx *ctx)
                 if(entry.value + value_size > map->len) {
                     cli_warnmsg("cli_parsetiff: TFD entry field %u exceeds bounds of TIFF file [%llu > %llu]\n",
                                 i, (long long unsigned)(entry.value + value_size), (long long unsigned)map->len);
-                    return cli_append_virus(ctx, "Heuristic.TIFF.OutOfBoundsAccess");
+                    return cli_append_virus(ctx, "Heuristics.TIFF.OutOfBoundsAccess");
                 }
             }
         }
