@@ -1,6 +1,6 @@
 /*
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
- *  Copyright (C) 2007-2013 Sourcefire, Inc.
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2007-2008 Sourcefire, Inc.
  *
  *  Authors: Alberto Wu
  *
@@ -28,7 +28,7 @@
 
 #include "others.h"
 
-typedef int (*zip_cb)(int fd, cli_ctx *ctx);
+typedef int (*zip_cb)(int fd, const char *filepath, cli_ctx *ctx);
 #define zip_scan_cb cli_magic_scandesc
 
 #define MAX_ZIP_REQUESTS 10
